@@ -31,9 +31,7 @@ END
 
 my $dir = "$Bin/../db";
 my $dbfile = "$dir/forum.db";
-my $modules = "$dir/modules.txt";
 unlink $dbfile if -e $dbfile;
-unlink $modules if -e $modules;
 mkdir $dir if not -e $dir;
 CPAN::Forum::DBI->myinit($dbfile);
 CPAN::Forum::DBI->init_db($dbfile);
@@ -43,7 +41,7 @@ chmod 0755, $dbfile;
 my $from = delete $opt{from};
 CPAN::Forum::Configure->create({field => 'from', value => $from});
 
-my $user = CPAN::Forum::Users->create({id => 1, %opt});
+my $user = CPAN::Forum::Users->create({id => 1, update_on_new_user => 1, %opt});
 $user->password($opt{password});
 $user->update;
 CPAN::Forum::Usergroups->create({id => 1, name => "admin"});
