@@ -2,7 +2,7 @@ package CPAN::Forum;
 use strict;
 use warnings;
 
-our $VERSION = "0.10";
+our $VERSION = "0.11";
 
 use base "CGI::Application";
 use CGI::Application::Plugin::Session;
@@ -177,8 +177,35 @@ Run:
 For some of the tests you'll have to set the CPAN_FORUM_URL environment 
 variable to the URL where you installed the forum.
 
+
+
 =head2 Changes
 
+v0.11
+  Search for users
+  Unite the serch methods
+  Accept both upper-case and lower-case HTML tags and turn them all to lower 
+    case tags when displaying
+  Accept <a href=> tags for http and mailto
+  Admin page
+  Admin can change "From" e-address
+  Enable <i>, <b> <br> and <a ..> with <p></p> pairs
+  Remove the selection box from the post interface as it was not used there.
+  Put the search form on the home page as well.
+  Admin can change e-mail address of any user
+  Add paging
+ 
+
+v0.10_02
+  <p>, <br> enabled
+  Add link to Kobes Search
+  Improve full text search for posts
+  Add capability to search for module names
+
+
+v0.10
+- markup improved, bugs fixed
+
 v0.09_05
 - POD cleanup (Shlomi Fish)
 
@@ -198,15 +225,28 @@ at both ends of the typed in word.
 
 =head2 TODO
 
+- Enable people to subscribe to all messages or all thread starters or all followups
+  (probably group = 0 will do it)
+- Add announcement service (check for new versions of modules and send e-mail
+  to those whom are interested in announcements.
+- Put time ellapsed since post instead of date of post, (3 min. 6 hours 3 days ago)
+  make this configurabel: User can say after N days passed the date should show,
+  before that the ellapsed time
+- Admin: hide a posting
+- Admin: delete a user
+- Admin: add a new module manually
+- Script that populates databse should not lock the whole database for a long time
+  Maybe it should fetch all the data to memory and work there.
+- make paging available responses 1..10, 11.20, etc, 
+
+
+
 - Decide on Basic Markup language and how to extend for shortcuts opening tag
 for code:  <code[^>]*>  but right now only <code> should be accepted closing
 tag for code:  </code>
 
 - check all submitted fields (restrict posting size to 10.000 Kbyte ?
-- Make the site look nicer (HTML and css work)
 - Improve text and explanations.
-- Improve Legal statement, look at other sites.
-
 
 clean up documentation
 
@@ -232,8 +272,6 @@ address from auth he might not want to give, for this case we should have our
 way to update the locally updated username, full name and validated e-mail
 address.
   
---- For XYZ we have to see how they work
-
 -- For local credentials we need the user to give us 
 username/password/fullname and validated e-mail address.
 
@@ -243,11 +281,6 @@ should use separate fields for usernames from various sources and when
 displayed we might prefix it auth:gabor, local:gabor etc.  Not nice, any better
 way ?
 
-- Add constraint checking to every field that the user can change by submitting
-information.  
-
-- Finalize markup
-
 Subject field:
 -  <= 50 chars
 -  Can contain any characters, we'll escape them when showing on the web site
@@ -291,11 +324,8 @@ one module to another module or group.
 Hmm, do I really need this ? maybe as I cannot just delete a user. (added a 
 status field that is not used currently)
 
-=head1 TODO Nice to have
+- Replace the /post/number link by /post/TITLE_OF_POST ???
 
-- Make sure adding a new module works fine
-
-- make paging available responses 1..10, 11.20, etc, 
 
 OK, so we have listing in places like
 
@@ -322,12 +352,8 @@ We'll also have some search facility that will be a post operation and
 From the forms we have post methods so no need for URL munging
 process_post  =>  (show previous post)? show editor + show preview
 
-=head1 TODO Next release only
-
 - make the page size (for paging) user configurable
 
-- Notify user
-
 =over 4
 
 A user can ask to be notified upon the following events per distribution.
@@ -471,6 +497,54 @@ variables) and then to construct a /cpan-forum/login/ /cpan-forum/register/
 etc. path. (or use relative URLs).
 
 
+=head2 TEMPLATES
+
+
+root templates:
+
+about.tmpl
+change_password.tmpl
+faq.tmpl
+groups.tmpl
+help.tmpl
+home.tmpl
+internal_error.tmpl
+login.tmpl
+module_search_form.tmpl
+module_select_form.tmpl
+mypan.tmpl
+notes.tmpl
+posts.tmpl
+pwreminder.tmpl
+search.tmpl
+users.tmpl
+register.tmpl
+threads.tmpl
+
+
+every root template should INCLUDE      -> head.tmpl navigation.tmpl footer.tmpl
+
+groups.tmpl            -> links.tmpl listing.tmpl     (list of messages within one group)
+home.tmpl              -> listing.tmpl                (list of messages in all the site)
+posts.tmpl             -> links.tmpl message.tmpl message.tmpl editor.tmpl  
+                                                      (single message 
+													  with or without the editor
+													  with or without a preview pane)
+search.tmpl            -> listing.tmpl                (list of messages resulted from search)
+threads.tmpl           -> links.tmpl
+users.tmpl             -> listing.tmpl                (list of messages of one user)
+
+
+Non root templates:
+links.tmpl      - a bunch of links to search.cpan.org and similar places (specific to one distro)
+listing.tmpl    - can show the titles of many messages 
+message.tmpl    - can show one message (or a preview message)
+naviagtion.tmpl -
+head.tmpl       -
+footer.tmpl     -
+
+Use this for mapping:
+grep INCLUDE *| grep -v navigation.tmpl | grep -v footer.tmpl | grep -v head.tmpl
 
 =head1 METHODS
 
@@ -542,6 +616,10 @@ my @free_modes = qw(home
 my @restricted_modes = qw(
 			new_post process_post
 			mypan 
+			admin
+			admin_process
+			admin_edit_user
+			admin_edit_user_process
 			response_form 
 			module_search
 			selfconfig change_password update_subscription); 
@@ -555,6 +633,8 @@ my @urls = qw(
 	threads dist users 
 	response_form 
 	faq 
+	admin
+	admin_edit_user
 	mypan selfconfig 
 	search all rss); 
 
@@ -609,7 +689,8 @@ sub cgiapp_prerun {
 		
 	}
 
-	$self->log->debug("Current runmode:  $rm");
+	$self->log->debug("Current runmode:  $rm"); 
+	$self->log->debug("Current user:  " . ($self->session->param("username") || ""));
 
 	return if grep {$rm eq $_} @free_modes;
 	#return if not grep {$rm eq $_} @restricted_modes;
@@ -633,7 +714,7 @@ rm=something
 
 sub autoload {
 	my $self = shift;
-	$self->internal_error;
+	$self->internal_error();
 }
 
 
@@ -645,53 +726,47 @@ most recent posts.
 =cut
 sub home {
 	my $self = shift;
+	my $q = $self->query;
 	
 	my $t = $self->load_tmpl("home.tmpl",
 		loop_context_vars => 1,
 	);
 	
-	my $from = ${$self->param("path_parameters")}[1] || 0;
-	my $cnt  = ${$self->param("path_parameters")}[2] || $limit;
-	$t->param(messages => $self->build_listing(
-			scalar CPAN::Forum::Posts->retrieve_latest($from+$cnt),
-			CPAN::Forum::Posts->count_all(),
-			));
-
+	my $page = $q->param('page') || 1;
+	$self->_search_results($t, {where => {}, page => $page, per_page => $limit});
 	$t->output;
 }
 
+sub _search_results {
+	my ($self, $t, $params) = @_;
+	
+	my $pager   = CPAN::Forum::Posts->mysearch($params);
+	my @results = $pager->search_where();
+    my $total   = $pager->total_entries;
+	$self->log->debug("number of entries: total=$total");
+    #$self->session->param('per_page'     => $per_page);
+    #$self->session->param('current_page' => $pager->current_page);
+	my $data = $self->build_listing(\@results);
+
+    $t->param(messages       => $data);
+    $t->param(total          => $total);
+    $t->param(previous_page  => $pager->previous_page);
+    $t->param(next_page      => $pager->next_page);
+    $t->param(first_entry    => $pager->first);
+    $t->param(last_entry     => $pager->last);
+    $t->param(first_page     => 1)                      if $pager->current_page != 1;
+    $t->param(last_page      => $pager->last_page)      if $pager->current_page != $pager->last_page;
+}
+
 sub all {
 	home(@_);
 }
 
-=head2 build_listing
-
-Receives a CPAN::Forum::Posts iterator and optionally two numbers
-Builds an array of hashes from all the posts or those in the given range
-and returns the array reference.
-
-=cut
-
 sub build_listing {
-	my ($self, $it, $total) = @_;
-	$self->log->debug("build_listing: total=$total");
-	
-	my $from = ${$self->param("path_parameters")}[1] || 0;
-	my $cnt  = ${$self->param("path_parameters")}[2] || $limit;
-	my $to   = $from+$cnt-1;
+	my ($self, $it) = @_;
 	
 	my @resp;
-	if ($to) {
-		$it = $it->slice($from, $to);
-	}
-
-	#my $start = $from % $cnt;
-	
-	
-	
-
-	while (my $post = $it->next) {
-		#(my $dashgroup = $post->gid) =~ s/::/-/g;
+	foreach my $post (@$it) {
 		my $thread_count = CPAN::Forum::Posts->sql_count_thread($post->thread)->select_val;
 		push @resp, {
 			subject      => _subject_escape($post->subject), 
@@ -710,6 +785,7 @@ sub build_listing {
 	return \@resp;
 }
 
+
 =head2 redirect_home
 
 Just to easily redirect to the home page
@@ -732,10 +808,17 @@ sub about {
 	my $self = shift;
 	my $t = $self->load_tmpl("about.tmpl");
 	
-	$t->param(distro_cnt => CPAN::Forum::Groups->count_all());
-	$t->param(posts_cnt  => CPAN::Forum::Posts->count_all());
-	$t->param(users_cnt  => CPAN::Forum::Users->count_all());
-	$t->param(version    => $VERSION);
+	$t->param(distro_cnt        => CPAN::Forum::Groups->count_all());
+	$t->param(posts_cnt         => CPAN::Forum::Posts->count_all());
+	$t->param(users_cnt         => CPAN::Forum::Users->count_all());
+	$t->param(subscription_cnt  => CPAN::Forum::Subscriptions->count_all());
+	$t->param(version           => $VERSION);
+	# number of posts per group name, can create some xml feed from it that can
+	# be used by search.cpan.org and Kobes to add a number of posts next to the link
+	#select count(*),groups.name from posts, groups where groups.id=gid group by gid;
+	#
+	#count posts for a specific group:
+	#select count(*) from posts, groups where groups.id=gid and groups.name="CPAN-Forum";
 
 	$t->output;
 }
@@ -755,9 +838,15 @@ Maybe this one should also receive the error message and print it to the log fil
 =cut
 
 sub internal_error {
-	my ($self, $msg) = @_;
-	cluck $msg if $msg;
+	my ($self, $msg, $tag) = @_;
+	if ($msg) {
+		$msg .= " REFERER: $ENV{HTTP_REFERER}" if $ENV{HTTP_REFERER};
+		warn $msg;
+		$self->log->debug($msg);
+	}
 	my $t = $self->load_tmpl("internal_error.tmpl");
+	$t->param($tag => 1) if $tag;
+	$t->param(generic => 1) if not $tag;
 	$t->output;
 }
 
@@ -776,6 +865,7 @@ sub load_tmpl {
 	$t->param("loggedin" => $self->session->param("loggedin") || "");
 	$t->param("username" => $self->session->param("username") || "anonymous");
 	$t->param("test_site_warning" => -e $self->param("ROOT") . "/config_test_site");
+	$t->param("admin" => $self->session->param('admin'));
 	return $t;
 }
 # config_fake_login  (not used currently)
@@ -821,12 +911,20 @@ sub login_process {
 	}
 
 	my $session = $self->session;
+	$session->param(admin     => 0); # make sure it is clean
+
 	$session->param(loggedin  => 1);
 	$session->param(username  => $user->username);
 	$session->param(uid       => $user->id);
 	$session->param(fname     => $user->fname); # TODO
 	$session->param(lname     => $user->lname); # TODO
 	$session->param(email     => $user->email);
+	foreach my $g (CPAN::Forum::Usergroups->search_ugs($user->id)) {
+		$self->log->debug("UserGroups: " . $g->name);
+		if ($g->name eq "admin") {
+			$session->param(admin     => 1);
+		}
+	}
 
 	my $request = $session->param("request") || "";
 	$session->param("request" => "");
@@ -877,7 +975,7 @@ sub register_process {
 	}
 	
 	# TODO arbitrary nickname constraint, allow other nicknames ?
-	if ($q->param('nickname') !~ /^[a-z0-9]{4,10}$/) {
+	if ($q->param('nickname') !~ /^[a-z0-9]{1,10}$/) {
 		return $self->register({"bad_nickname" => 1});
 	}
 
@@ -958,6 +1056,7 @@ sub pwreminder {
 	);
 
 	$t->param($errs) if $errs;
+	$t->param($q->param('field') => 1);
 	return $t->output;
 }
 
@@ -965,16 +1064,12 @@ sub pwreminder {
 sub pwreminder_process {
 	my ($self) = @_;
 	my $q = $self->query;
-	if (not $q->param('nickname') and not $q->param('email')) {
+	my $field = $q->param('field');
+	if (not $field or $field !~ /^username|email$/ or not $q->param('value')) {
 		return $self->pwreminder({"no_data" => 1});
 	}
 
-	my $user;
-	if ($q->param('nickname')) {
-		($user) = CPAN::Forum::Users->search({username => $q->param('nickname')});
-	} else {
-		($user) = CPAN::Forum::Users->search({email    => $q->param('email')});
-	};
+	my ($user) = CPAN::Forum::Users->search({$field => $q->param('value')});
 	return $self->pwreminder({"no_data" => 1}) if not $user;
 
 	# TODO: put this text in a template
@@ -991,8 +1086,8 @@ http://$ENV{HTTP_HOST}/
 
 MSG
 
-	my ($field) = CPAN::Forum::Configure->search({field => "from"});
-	my $FROM = $field->value;
+	my ($from) = CPAN::Forum::Configure->search({field => "from"});
+	my $FROM = $from->value;
 	$self->log->debug("FROM field set to be $FROM");
 
 	my %mail = (
@@ -1108,11 +1203,12 @@ sub posts {
 	$self->log->debug("posts rm=$rm");
 
 	my $new_group = "";
-	my $new_group_id;
+	my $new_group_id = "";
 	
 	if ($rm eq "new_post") {
 		$new_group = ${$self->param("path_parameters")}[0] || "";
 		$new_group_id = $q->param('new_group') if $q->param('new_group');
+		$self->log->debug("A: new_group: '$new_group' and id: '$new_group_id'");
 		
 		if ($new_group) {
 			if ($new_group =~ /^([\w-]+)$/) {
@@ -1121,20 +1217,23 @@ sub posts {
 				if ($gr) {
 					$new_group_id = $gr->id;
 				} else {
-					cluck "Group '$new_group' was not in database when accessed PATH_INFO: '$ENV{PATH_INFO}'";
-					return $self->internal_error;
+					return $self->internal_error(
+						"Group '$new_group' was not in database when accessed PATH_INFO: '$ENV{PATH_INFO}'",
+						);
 				}
 			} else {
-				cluck "Bad regex for '$new_group' ? Accessed PATH_INFO: '$ENV{PATH_INFO}'";
-				return $self->internal_error;
+				return $self->internal_error(
+					"Bad regex for '$new_group' ? Accessed PATH_INFO: '$ENV{PATH_INFO}'",
+					);
 			}
 		} elsif ($new_group_id) {
 			my ($gr) = CPAN::Forum::Groups->retrieve($new_group_id);
 			if ($gr) {
 				$new_group = $gr->name;
 			} else {
-				cluck "Group '$new_group_id' was not in database when accessed PATH_INFO: '$ENV{PATH_INFO}'";
-				return $self->internal_error;
+				return $self->internal_error(
+					"Group '$new_group_id' was not in database when accessed PATH_INFO: '$ENV{PATH_INFO}'",
+				);
 			}
 		} elsif ($q->param('q')) {
 			# process search later	
@@ -1142,12 +1241,15 @@ sub posts {
 			# TODO should be called whent the module_search is ready
 			return $self->module_serach_form();
 		}
+		$self->log->debug("B: new_group: '$new_group' and id: '$new_group_id'");
 	}
 	if ($rm eq "process_post") {
-		$new_group_id = $q->param("new_group");
-		return $self->internal_error(
-			"Missing new_group_id. Accessed PATH_INFO: '$ENV{PATH_INFO}'")
-			if not $new_group_id;
+		$new_group_id = $q->param("new_group_id");
+		if (not $new_group_id) {
+			return $self->internal_error(
+				"Missing new_group_id. Accessed PATH_INFO: '$ENV{PATH_INFO}'",
+				);
+		}
 
 		if ($new_group_id =~ /^(\d+)$/) {
 			$new_group_id = $1;
@@ -1155,18 +1257,17 @@ sub posts {
 			if ($grp) {
 				$new_group = $grp->name;
 			} else {
-				return $self->internal_error("Bad value for new_group (id) '$new_group_id' ? Accessed PATH_INFO: '$ENV{PATH_INFO}'");
+				return $self->internal_error(
+					"Bad value for new_group (id) '$new_group_id' ? Accessed PATH_INFO: '$ENV{PATH_INFO}'",
+					);
 			} 
 		} else {
-			return $self->internal_error("Bad value for new_group (id) '$new_group_id' ? Accessed PATH_INFO: '$ENV{PATH_INFO}'");
+			return $self->internal_error(
+				"Bad value for new_group (id) '$new_group_id' ? Accessed PATH_INFO: '$ENV{PATH_INFO}'",
+				);
 		}
 	}
-	#warn $new_group;
-	#warn $new_group_id;
-
-	#$new_group =~ s/-/::/g;
-	#(my $dashgroup = $new_group) =~ s/::/-/g;
-	#$t->param(dashgroup => $dashgroup);
+	$self->log->debug("C: new_group: '$new_group' and id: '$new_group_id'");
 
 	my $title = ""; # of the page
 	my $editor = 0;
@@ -1181,8 +1282,9 @@ sub posts {
 	if ($id) { # Show post
 		my $post = CPAN::Forum::Posts->retrieve($id);
 		if (not $post) {
-			cluck "PATH_INFO: $ENV{PATH_INFO}";
-			return $self->internal_error;
+			return $self->internal_error(
+				"PATH_INFO: $ENV{PATH_INFO}",
+				);
 		}
 		my $thread_count = CPAN::Forum::Posts->sql_count_thread($post->thread)->select_val;
 		if ($thread_count > 1) {
@@ -1206,7 +1308,11 @@ sub posts {
 		$new_group        = $post->gid->name;
 		$new_group_id     = $post->gid->id;   	
 	}
-	$t->param("group_selector" => $self->_group_selector($new_group, $new_group_id));
+	$self->log->debug("D: new_group: '$new_group' and id: '$new_group_id'");
+	#$t->param("group_selector" => $self->_group_selector($new_group, $new_group_id));
+	$t->param(new_group    => $new_group);
+	$t->param(new_group_id => $new_group_id);
+	$t->param(new_text     => CGI::escapeHTML($q->param("new_text")));
 	
 	# for previewing purposes:
 	# This is funky, in order to use the same template for regular show of a message and for
@@ -1251,8 +1357,8 @@ sub process_post {
 		($parent_post) = CPAN::Forum::Posts->search(id => $parent);
 		push @errors, "bad_thing"  if not $parent_post;
 	} else {       # assume new post
-		if ($q->param("new_group")) {
-			push @errors, "bad_group"  if not CPAN::Forum::Groups->search(id => $q->param("new_group"));
+		if ($q->param("new_group_id")) {
+			push @errors, "bad_group"  if not CPAN::Forum::Groups->search(id => $q->param("new_group_id"));
 		} else {
 			push @errors, "no_group";
 		}
@@ -1289,15 +1395,16 @@ sub process_post {
 		return $self->posts(["preview"]);
 	}
 	if ($button ne "Submit") {
-		warn "Someone sent in a button called '$button'";
-		return $self->internal_error;
+		return $self->internal_error(
+			"Someone sent in a button called '$button'",
+			);
 	}
 
 	my $pid;
 	eval {
 		my $post = CPAN::Forum::Posts->create({
 			uid     => $self->session->param("username"),
-			gid     => $parent_post ? $parent_post->gid : $q->param("new_group"),
+			gid     => $parent_post ? $parent_post->gid : $q->param("new_group_id"),
 			subject => $q->param("new_subject"),
 			text    => $new_text,
 			date    => time,
@@ -1306,16 +1413,14 @@ sub process_post {
 		$post->parent($parent) if $parent_post;
 		$post->update;
 		$pid = $post->id;
-		#warn $parent_post ? $parent_post->gid : $q->param("new_group");
-		#warn "PG:" . $post->gid;
 	};
 	if ($@) {
 		#push @errors, "subject_too_long" if $@ =~ /subject_too_long/;
 		#warn $CPAN::Forum::Post::lasterror if $@ =~ /text_format/;
 		if (not @errors) {
-			warn "UNKNOWN_ERROR: $@";
-			cluck "PATH_INFO: $ENV{PATH_INFO}";
-			return $self->internal_error;
+			return $self->internal_error(
+				"PATH_INFO: '$ENV{PATH_INFO}'\nUNKNOWN_ERROR: $@",
+			);
 		}
 		return $self->posts(\@errors);
 	}
@@ -1392,8 +1497,9 @@ sub threads {
 
 	my @posts = CPAN::Forum::Posts->search(thread => $id);
 	if (not @posts) {
-		cluck "PATH_INFO: $ENV{PATH_INFO}";
-		return $self->internal_error;
+		return $self->internal_error(
+			"PATH_INFO: $ENV{PATH_INFO}",
+			);
 	}
 
 	my @posts_html;
@@ -1438,31 +1544,30 @@ sub dist {
 	if ($group =~ /^([\w-]+)$/) {
 		$group = $1;
 	} else {
-		warn "Probably bad regex when checking group name for $group called in $ENV{PATH_INFO}";
-		return $self->internal_error();
+		return $self->internal_error(
+			"Probably bad regex when checking group name for $group called in $ENV{PATH_INFO}",
+			);
 	}
 
 	my ($gr) = CPAN::Forum::Groups->search(name => $group);
 	if (not $gr) {
-		warn "Invalid group $group called in $ENV{PATH_INFO}";
-		return $self->internal_error();
+		return $self->internal_error(
+			"Invalid group $group called in $ENV{PATH_INFO}",
+			"no_such_group",
+			);
 	}
 	my $gid = $gr->id;
 	if ($gid =~ /^(\d+)$/) {
 		$gid = $1;
 	} else {
-		warn "Invalid gid received $gid called in $ENV{PATH_INFO}";
-		return $self->internal_error();
+		return $self->internal_error(
+			"Invalid gid received $gid called in $ENV{PATH_INFO}",
+			);
 	}
 
-
-	$t->param(messages => $self->build_listing(
-			scalar CPAN::Forum::Posts->search(gid => $gid, {order_by => 'date DESC'}),
-			CPAN::Forum::Posts->sql_count_where("gid", $gid)->select_val,
-			));
-
+	my $page = $q->param('page') || 1;
+	$self->_search_results($t, {where => {gid => $gid}, page => $page, per_page => $limit});
 	$t->output;
-
 }
 
 =head2 users
@@ -1480,8 +1585,9 @@ sub users {
 	$username = ${$self->param("path_parameters")}[0];
 
 	if (not $username) {
-		cluck "No username: PATH_INFO: $ENV{PATH_INFO}";
-		return $self->internal_error;
+		return $self->internal_error(
+			"No username: PATH_INFO: $ENV{PATH_INFO}",
+			);
 	}
 
 	my $t = $self->load_tmpl("users.tmpl",
@@ -1492,8 +1598,9 @@ sub users {
 	my ($user) = CPAN::Forum::Users->search(username => $username);
 
 	if (not $user) {
-		warn "Non existing user was accessed: $ENV{PATH_INFO}";
-		return $self->internal_error;
+		return $self->internal_error(
+			"Non existing user was accessed: $ENV{PATH_INFO}",
+			);
 	}
 
 
@@ -1507,10 +1614,8 @@ sub users {
 	$t->param(this_fullname => $fullname);
 	$t->param(title => "Information about $username");
 
-	$t->param(messages => $self->build_listing(
-			scalar CPAN::Forum::Posts->search(uid => $username, {order_by => 'date DESC'}),
-			CPAN::Forum::Posts->sql_count_where("uid", $username)->select_val,
-			));
+	my $page = $q->param('page') || 1;
+	$self->_search_results($t, {where => {uid => $username}, page => $page, per_page => $limit});
 	$t->output;
 }
 
@@ -1553,8 +1658,9 @@ sub mypan {
 	my ($user) = CPAN::Forum::Users->search(username => $username);
 
 	if (not $user) {
-		warn "Trouble accessing personal information of: '$username' $ENV{PATH_INFO}";
-		return $self->internal_error;
+		return $self->internal_error(
+			"Trouble accessing personal information of: '$username' $ENV{PATH_INFO}",
+			);
 	}
 	my $fullname = "";
 	$fullname .= $user->fname if $user->fname;
@@ -1577,8 +1683,9 @@ sub mypan {
 		my $group = $params[1];
 		my ($grp) = CPAN::Forum::Groups->search(name => $group);
 		if (not $grp) {
-			warn "Accessing $ENV{PATH_INFO}\n";
-			return $self->internal_error;
+			return $self->internal_error(
+				"Accessing $ENV{PATH_INFO}\n",
+			);
 		}
 		$gids = $grp->id;
 		my ($s) = CPAN::Forum::Subscriptions->search(uid => $user->id, gid => $grp->id);
@@ -1629,7 +1736,7 @@ sub update_subscription {
 	#warn $q->param("gids");
 	my @gids = split /,/, $q->param("gids");
 	if (not @gids) {
-		return $self->internal_error;
+		return $self->internal_error();
 	}
 
 	my $username = $self->session->param("username");
@@ -1670,7 +1777,6 @@ sub notes {
 }
 
 
-# partially written code to select a module name
 sub module_search {
 	my ($self) = @_;
 
@@ -1688,6 +1794,7 @@ sub module_search {
 		return $self->module_serach_form(['invalid_search_term']);
 	}
 	$self->log->debug("group name search term: $txt");
+	$txt =~ s/::/-/g;
 	$txt = '%' . $txt . '%';
 	
 	my $it =  CPAN::Forum::Groups->search_like(name => $txt);
@@ -1710,34 +1817,148 @@ sub module_search {
 	$t->output;
 }
 
-=head2 search
-
-Search form and processor.
-
-=cut
-
 sub search {
-	my $self = shift;
+	my ($self) = @_;
+	my $q      = $self->query;
+	my $name   = $q->param("name")    || '';
+	my $what   = $q->param("what")    || '';
 
-	my $q = $self->query;
-	my $txt = $q->param("q");
+	# kill the taint checking (why do I use taint checking if I kill it then ?)
+	if ($name =~ /(.*)/) { $name    = $1; }
+	$name =~ s/::/-/g if $what eq "module";
 	
 	my $t = $self->load_tmpl("search.tmpl",
 		associate => $q,
 		loop_context_vars => 1,
 	);
+	my $it;
 
-	# kill the taint checking (why do I use taint checking if I kill it then ?)
-	if ($txt =~ /(.*)/) {
-		$txt = $1;
+	if (not $what and not $name) {
+		$what = $self->session->param('search_what');
+		$name = $self->session->param('search_name');
 	}
 
-	if ($txt) {
-		my $it =  CPAN::Forum::Posts->search_like(text => '%' . $txt . '%');
-		my $cnt = CPAN::Forum::Posts->sql_count_like("text", '%' . $txt . '%')->select_val;
-		$t->param(messages => $self->build_listing($it,$cnt));
+	$self->session->param(search_what => $what);
+	$self->session->param(search_name => $name);
+
+	if ($what and $name) {
+		if ($what eq "module") {
+			my @things;
+			my $it =  CPAN::Forum::Groups->search_like(name => $name . '%');
+			while (my $group  = $it->next) {
+				push @things, {name => $group->name};
+			}
+			$t->param(groups => \@things);
+			$t->param($what => 1);
+		} elsif ($what eq "user") {
+			my @things;
+			my $it =  CPAN::Forum::Users->search_like(username => '%' . lc($name) . '%');
+			while (my $user  = $it->next) {
+				push @things, {username => $user->username};
+			}
+			$t->param(users => \@things);
+			$t->param($what => 1);
+		} else {
+			my %where;
+			if ($what eq "subject") { %where = (subject => {'LIKE', '%' . $name . '%'}); }
+			if ($what eq "text")    { %where = (text    => {'LIKE', '%' . $name . '%'}); }
+			$self->log->debug("Search 1: " . join "|", %where);
+			if (%where) {
+
+				$self->log->debug("Search 2: " . join "|", %where);
+
+				my $page = $q->param('page') || 1;
+				$self->_search_results($t, {where => \%where, page => $page, per_page => $limit});
+
+				$t->param($what => 1);
+			}
+		}
+	}
+	$t->output;
+}
+
+sub admin_edit_user_process {
+	my ($self) = @_;
+	if (not $self->session->param("admin")) {
+		return $self->internal_error("", "restricted_area");
+	}
+	my $q = $self->query;
+	my $email = $q->param('email');
+	my $uid   = $q->param('uid'); # TODO error checking here !
+
+	$self->log->debug("admin_edit_user_process uid: '$uid'");
+	my ($person) = CPAN::Forum::Users->retrieve($uid);
+	if (not $person) {
+		return $self->internal_error("", "no_such_user");
+	}
+	$person->email($email);
+	$person->update;
+
+	$self->admin_edit_user($person->username, ['done']);
+}
+
+sub admin_edit_user {
+	my ($self, $username, $errors) = @_;
+	if (not $self->session->param("admin")) {
+		return $self->internal_error("", "restricted_area");
+	}
+	my $q = $self->query;
+	if (not $username) {
+		$username = ${$self->param("path_parameters")}[0] || '';
+	}
+	$self->log->debug("admin_edit_user username: '$username'");
+
+	my ($person) = CPAN::Forum::Users->search(username => $username);
+	if (not $person) {
+		return $self->internal_error("", "no_such_user");
 	}
 
+	my $t = $self->load_tmpl("admin_edit_user.tmpl");
+	$t->param(this_username => $username);
+	$t->param(email => $person->email);
+	$t->param(uid   => $person->id);
+
+	if ($errors and ref($errors) eq "ARRAY") {
+		$t->param($_ => 1) foreach @$errors;
+	}
+
+	$t->output;
+
+}
+
+sub admin_process {
+	my ($self) = @_;
+	if (not $self->session->param("admin")) {
+		return $self->internal_error("", "restricted_area");
+	}
+	my $q = $self->query;
+
+	if (my ($conf) = CPAN::Forum::Configure->search(field => 'from')) {
+		$self->log->debug("Old FROM field was " . $conf->value);
+		$conf->value($q->param('from'));
+		$self->log->debug("New FROM field set to be " . $q->param('from'));
+		$conf->update;
+	} else {
+		$self->log->fatal("Could not find from field !!");
+	}
+
+	my $t = $self->load_tmpl("admin.tmpl");
+	$t->param(updated => 1);
+	$t->output;
+}
+
+
+sub admin {
+	my ($self) = @_;
+	if (not $self->session->param("admin")) {
+		return $self->internal_error("", "restricted_area");
+	}
+	my %data;
+	foreach my $c (CPAN::Forum::Configure->retrieve_all()) {
+		$data{$c->field} = $c->value;
+	}
+	my $t = $self->load_tmpl("admin.tmpl");
+	$t->param(%data);
 	$t->output;
 }
 
@@ -1888,11 +2109,13 @@ sub help {
 	$_[0]->load_tmpl("help.tmpl")->output;
 }
 
+
 1;
 
 =head1 ACKNOWLEDGEMENTS
 
-Thanks to Offer Kaye for his initial help with HTML and CSS.  Thanks to all
+Thanks to Offer Kaye for his initial help with HTML and CSS.  Thanks
+to Shlomi Fish for some patches. Thanks to all
 the people who develop and maintain the underlying technologies.  See
 L<http://www.cpanforum.com/about/> for a list of tools we used.  In addition to
 Perl of course.
