@@ -23,6 +23,19 @@ sub retrieve_latest {
 	return $class->sth_to_objects($class->sql_latest($count));
 }
 
+sub mysearch {
+	my ($self, $params) = @_;
+
+	my %where  = %{$params->{where}};
+	%where = (1 => 1) if not %where;
+
+	my $pager = __PACKAGE__->pager(
+		where         => \%where,
+		per_page      => $params->{per_page} || 10,
+		page          => $params->{page}     || 1,
+		order_by      => $params->{order_by} || "id DESC",
+	);
+}
 
 1;
  
